package com.sun.security.sasl.gsskerb;

import javax.security.sasl.SaslServer;
import javax.security.sasl.SaslException;
import com.sun.security.sasl.util.PolicyUtils;
import javax.security.sasl.SaslClient;
import javax.security.auth.callback.CallbackHandler;
import java.util.Map;
import javax.security.sasl.SaslServerFactory;
import javax.security.sasl.SaslClientFactory;

public final class FactoryImpl implements SaslClientFactory, SaslServerFactory
{
    private static final String[] myMechs;
    private static final int[] mechPolicies;
    private static final int GSS_KERB_V5 = 0;
    
    public FactoryImpl() {
    }
    
    @Override
    public SaslClient createSaslClient(String[] array, String s, String s2, String s3, Map<String, ?> props, CallbackHandler callbackHandler) throws SaslException {
        int i = 0;
        while (i < array.length) {
            if (array[i].equals(FactoryImpl.myMechs[0]) && PolicyUtils.checkPolicy(FactoryImpl.mechPolicies[0], props)) {
                return new GssKrb5Client(s, s2, s3, props, callbackHandler);
            }
            ++i;
        }
        return null;
    }
    
    @Override
    public SaslServer createSaslServer(String s, String s2, String s3, Map<String, ?> props, CallbackHandler callbackHandler) throws SaslException {
        if (s.equals(FactoryImpl.myMechs[0]) && PolicyUtils.checkPolicy(FactoryImpl.mechPolicies[0], props)) {
            if (callbackHandler == null) {
                throw new SaslException("Callback handler with support for AuthorizeCallback required");
            }
            return new GssKrb5Server(s2, s3, props, callbackHandler);
        }
        else {
            return null;
        }
    }
    
    @Override
    public String[] getMechanismNames(Map<String, ?> props) {
        return PolicyUtils.filterMechs(FactoryImpl.myMechs, FactoryImpl.mechPolicies, props);
    }
    
    static {
        myMechs = new String[] { "GSSAPI" };
        mechPolicies = new int[] { 19 };
    }
}
